﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.Reflection;

using IdeaBlade.EntityModel;
using IdeaBlade.Core;
using System.Data;
using IdeaBlade.Core.Reflection;

namespace SampleSupport {
  public class TreeViewGenerator {
   
    private TreeViewGenerator() {

    }

    public static TreeViewGenerator Instance {
      get { return __instance; }
    }

    public int ExpandDepth { get; set; }
    public int MaxLoadDepth { get; set; }

    public  TreeNode GetTreeNode(object o) {
  
      if (o == null) {
        return GetNullNode();
      }
          
      if (o is Exception) {
        return GetExceptionNode((Exception)o);
      }

      if (o is DateTime) {
        return GetDateTimeNode((DateTime)o);
      }

      if (o is String) {
        return GetStringNode((string)o);
      }

      if (o is Type) {
        return GetTypeNode((Type)o);
      }

      if (o is byte[]) {
        return GetByteArrayNode((byte[])o);
      }

      if (o is ValueType) {
        return GetValueTypeNode(o);
      }

      if (o is PropertyRef) {
        return GetPropertyRefNode((PropertyRef)o);
      }

      if (o is IEnumerable) {
        return GetEnumerableNode(o);
      }

      return GetNestedObjectNode(o);
    }

    public void ExpandTreeNode(TreeNode parentNode) {
      if (parentNode.Nodes.Count > 1) return;

      var o = parentNode.Tag;
      if (o == null) return;
      parentNode.Tag = null;  // insure we don't eval again;
      parentNode.Nodes.Clear();
      IEnumerable<Member> members;
      if (o is PropertyRef) {
        var propRef = o as PropertyRef;
        var eval = propRef.Evaluate();
        if (eval is IEnumerable) {
          parentNode.SelectedImageIndex = parentNode.ImageIndex = 4;
          var itermembers = (from object element in (eval as IEnumerable)
                             select new Member { Name = null, Value = element }).ToList();

          AttachChildren(parentNode, itermembers); ;
          return;
        } else {
          members = BuildMembers(propRef.Evaluate());
        }
      } else {
        members = BuildMembers(o);
      }
      AttachChildren(parentNode, members); ;

    }

    private TreeNode GetNestedObjectNode(object o) {
      var parentNode = BuildDefaultParentNode(o);
      parentNode.SelectedImageIndex = parentNode.ImageIndex = 1;
      parentNode.Tag = o;
      parentNode.Nodes.Add("dummyNode");  // needed to show '+' sign
      // currently only updates text for entities - don't know what else can get in here
      UpdateNodeText(o, parentNode);
      return parentNode;
    }

    private TreeNode GetPropertyRefNode(PropertyRef pref) {
      TreeNode parentNode = new TreeNode(GetNiceTypeName(pref.Property.PropertyType) + " (Lazy)");
      parentNode.Tag = pref;
      parentNode.Nodes.Add("dummyNode");  // needed to show '+' sign
      return parentNode;
    }

    private TreeNode GetEnumerableNode(object o ) {
      var parentNode = BuildDefaultParentNode(o);
      parentNode.SelectedImageIndex = parentNode.ImageIndex = 4;
      var members = (from object element in (o as IEnumerable)
                     select new Member { Name = null, Value = element }).ToList();

      AttachChildren(parentNode, members); ;
      parentNode.Text += " (" + members.Count + " item" + (members.Count != 1 ? "s" : "") + ")";
      return parentNode;
    }

    private TreeNode BuildDefaultParentNode(object o) {
      TreeNode parentNode = new TreeNode(GetNiceTypeName(o.GetType()));

      if (o is ObjectCollectionCache) {
        parentNode.Text = GetNiceTypeName(((ObjectCollectionCache)o).OriginalType);
      }
      return parentNode;
    }

    private static void UpdateNodeText(object o, TreeNode parentNode) {
      Entity ewk = o as Entity;
      if (ewk != null) {
        StringBuilder sb = new StringBuilder(parentNode.Text.Length + 20);
        sb.Append(parentNode.Text);
        sb.Append("(");

        bool isFirst = true;
        foreach (var prop in ewk.EntityAspect.EntityMetadata.KeyProperties) {
          if (isFirst) {
            isFirst = false;
          } else {
            sb.Append(", ");
          }
          string pkColumnName = prop.Name;
          sb.Append(pkColumnName);
          sb.Append("=");
          sb.Append(ewk.EntityAspect.GetValue(pkColumnName).ToString());
        }
        sb.Append(")");
        parentNode.Text = sb.ToString();
      }
    }

    private string GetNiceTypeName(Type t) {
      if (t.Name.Contains("AnonymousType"))
        return "(AnonymousType)";

      if (t.Name.Contains("SelectIterator"))
        return "(SelectIterator)";

      if (t.IsGenericType) {
        string name = t.Name;
        int backquote = name.IndexOf('`');
        if (backquote >= 0)
          name = name.Substring(0, backquote);

        StringBuilder sb = new StringBuilder();
        sb.Append(name);
        sb.Append("<");
        Type[] args = t.GetGenericArguments();
        for (int i = 0; i < args.Length; ++i) {
          if (i > 0) sb.Append(", ");
          sb.Append(GetNiceTypeName(args[i]));
        }
        sb.Append(">");
        return sb.ToString();
      }

      return t.Name;
    }



    public static IEnumerable<Member> BuildMembers(object o) {

      foreach (var member in o.GetType().GetMembers(BindingFlags.Public | BindingFlags.Instance)) {
        var property = member as PropertyInfo;
        if (property == null) continue;
        if (property.DeclaringType == typeof(Entity)) {
          if ( property.Name != "IsNullEntity" && property.Name != "EntityMetadata") continue;
        }
        if (property.DeclaringType == typeof(ComplexObject)) continue;
        if (property.Name.Equals("Item")) continue; // indexer

        if (typeof(Entity).IsAssignableFrom(property.PropertyType) 
          && property.Name.Equals("ScalarValue")) {
          yield return new Member { Name = property.Name, Value = new PropertyRef(o, member as PropertyInfo) };
          continue;
        }

        if (typeof(Entity).IsAssignableFrom(property.PropertyType)
        && property.Name.Equals("ListValue")) {
          yield return new Member { Name = property.Name, Value = new PropertyRef(o, member as PropertyInfo) };
          continue;
        }

        // if property returns a RelatedEntityList<>
        if (typeof(IEnumerable).IsAssignableFrom(property.PropertyType) 
          && property.PropertyType.IsGenericType ) {
          var elementType = TypeFns.GetGenericArgument(property.PropertyType);
          var genPropertyType = property.PropertyType.GetGenericTypeDefinition();
          if (genPropertyType.Equals(typeof(RelatedEntityList<>))) {
            yield return new Member { Name = property.Name, Value = new PropertyRef(o, member as PropertyInfo) };
            continue;
          }
        }

        var val = property.GetValue(o, null);
        if (val != null) {
          // if (typeof(Entity).IsAssignableFrom(val.GetType())) continue;
          if (typeof(EntityGroup).IsAssignableFrom(val.GetType())) continue;
          if (typeof(EntityQueryProxy).IsAssignableFrom(val.GetType())) continue;
          if (typeof(EntityManager).IsAssignableFrom(val.GetType())) continue;
          if (typeof(DataColumn).IsAssignableFrom(val.GetType())) continue;
          if (typeof(EntityProperty).IsAssignableFrom(val.GetType())) continue;
        }

        var m = new Member { Name = property.Name, Value = val };
        yield return m;
      }
    }

    private  void AttachChildren(TreeNode parentNode, IEnumerable<Member> members) {
      int counter = 0;

      foreach (Member m in members) {
        TreeNode valueNode;
        if (parentNode.Level >= MaxLoadDepth) {
          valueNode = new TreeNode("{ ... }");
        } else {
          valueNode = GetTreeNode(m.Value);
          if (m.Name != null) {
            valueNode.Text = m.Name + " = " + valueNode.Text;
          } else {
            valueNode.Text = "#" + counter++ + ": " + valueNode.Text;
          }
        }
        parentNode.Nodes.Add(valueNode);
      }

      if (parentNode.Level < ExpandDepth) {
        parentNode.Expand();
      }
    }

    private TreeNode GetDateTimeNode(DateTime dateTime) {
      return new TreeNode(dateTime.ToShortDateString());
    }

    private TreeNode GetNullNode() {
      return new TreeNode("null");
    }

    private TreeNode GetStringNode(string s) {
      return new TreeNode("\"" + s + "\"");
    }

    private TreeNode GetValueTypeNode(object o) {
      return new TreeNode(Convert.ToString(o));
    }

    private TreeNode GetExceptionNode(Exception ex) {
      TreeNode node = new TreeNode(ex.GetType().Name + ": " + ex.Message);
      node.Nodes.Add(new TreeNode("Source: " + ex.Source));
      return node;
    }

    private TreeNode GetTypeNode(Type t) {
      TreeNode node = new TreeNode(GetNiceTypeName(t));
      return node;
    }

    private TreeNode GetByteArrayNode(byte[] arr) {
      int length = Math.Min(arr.Length, 32);
      string t = "Byte[" + arr.Length + "] = " + BitConverter.ToString(arr, 0, length) + ((length != arr.Length) ? "..." : "");
      TreeNode node = new TreeNode(t);
      return node;
    }

    private static TreeViewGenerator __instance = new TreeViewGenerator();

    public class Member {
      public string Name;
      public object Value;
    }

    internal class PropertyRef {
      public PropertyRef(Object obj, PropertyInfo property) {
        BaseObject = obj;
        Property = property;
      }
      public Object BaseObject { get; private set; }
      public PropertyInfo Property { get; private set; }
      public Object Evaluate() {
        return Property.GetValue(BaseObject, null);
      }
    }
  }
}
