'Copyright (C) Microsoft Corporation.  All rights reserved.

Imports System.Linq
Imports DomainModel
Imports SampleSupport
Imports IdeaBlade.EntityModel

Namespace SampleQueries
  <Title("Linq To Entities"), Prefix("LinqToEntities")>
  Friend Class LinqToEntitiesSamples
	  Inherits NorthwindBasedSample

	#Region "Restriction Operators"

	<Category("Restriction Operators"), Title("Where - Simple 0"), Description("This sample uses WHERE to find all customers whose contact title is Sales Representative.")>
	Public Sub LinqToEntities0()
	  Dim query = From od In Context.OrderDetails
	              Where od.Product.ProductName = "Chai"
	              Select od
	  ObjectDumper.Write(query)
	End Sub

	<Category("Restriction Operators"), Title("Where - Simple 1"), Description("This sample uses WHERE to find all customers whose contact title is Sales Representative.")>
	Public Sub LinqToEntities1()
	  Dim query = From cust In Context.Customers
	              Where cust.ContactTitle = "Sales Representative"
	              Select cust
	  ObjectDumper.Write(query)
	End Sub

	<Category("Restriction Operators"), Title("Where - Simple 2"), Description("This sample uses WHERE to find all orders placed before 1997.")>
	Public Sub LinqToEntities2()
	  Dim dt As New Date(1997, 1, 1)
	  Dim query = From order In Context.Orders
	              Where order.OrderDate < dt
	              Select order

	  ObjectDumper.Write(query)
	End Sub

	<Category("Restriction Operators"), Title("Where - Simple 3"), Description("This sample uses WHERE to filter for Products that have stock below their reorder level and have a units on order of zero.")>
	Public Sub LinqToEntities3()
	  Dim query = From p In Context.Products
	              Where p.UnitsInStock < p.ReorderLevel AndAlso p.UnitsOnOrder = 0
	              Select p

	  ObjectDumper.Write(query)
	End Sub

	<Category("Restriction Operators"), Title("Where - Simple 4"), Description("This sample uses WHERE to filter out Products that have a UnitPrice less than 10.")>
	Public Sub LinqToEntities4()
	  Dim query = From p In Context.Products
	              Where p.UnitPrice < 10
	              Select p

	  ObjectDumper.Write(query)
	End Sub

	<Category("Restriction Operators"), Title("Where - Complex Type"), Description("This sample uses WHERE to find Employees in London.")>
	Public Sub LinqToEntities4a()
	  Dim query = From e In Context.Employees
	              Where e.Address.City = "London"
	              Select e

	  ObjectDumper.Write(query)
	End Sub
	<Category("Restriction Operators"), Title("Where - TypeOf"), Description("This sample uses WHERE to get previous employees.")>
	Public Sub LinqToEntities5()
	  Dim query = From e In Context.Employees
	              Where TypeOf e Is DomainModel.PreviousEmployee
	              Select e

	  ObjectDumper.Write(query)
	End Sub

	<Category("Restriction Operators"), Title("Any"), Description("This sample uses WHERE to get employees who handle the Boston territory.")>
	Public Sub LinqToEntities6()
	  Dim query = From e In Context.Employees
	              Where e.Territories.Any(Function(t) t.TerritoryDescription = "Boston")
	              Select e

	  ObjectDumper.Write(query)
	End Sub

	<Category("Restriction Operators"), Title("Any - 2"), Description("This sample uses any Customers who placed an order in 1997.")>
	Public Sub LinqToEntities7()
	  Dim query = From c In Context.Customers
	              Where c.Orders.Any(Function(o) o.OrderDate.HasValue = True AndAlso o.OrderDate.Value.Year = 1997)
	              Select c

	  ObjectDumper.Write(query)
	End Sub

	<Category("Restriction Operators"), Title("Any - 3"), Description("This sample uses ANY to check for any out-of-stock products.")>
	Public Sub LinqToEntities8()
      'Dim query = Context.Suppliers.Where(Function(s) s.Products.Any(Function(p) p.UnitsInStock = 0)).Select(Function(s) s)
      Dim query = Context.Suppliers.Where(Function(s) s.Products.Any(Function(p) CType(p.UnitsInStock = 0, Boolean))).Select(Function(s) s)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Restriction Operators"), Title("Any - Related Entities"), Description("This sample uses WHERE and ANY to get orders containing a product with a unit on order.")>
	Public Sub LinqToEntities9()
      Dim query = From o In Context.Orders
      Where (o.OrderDetails.Any(Function(od) CType(od.Product.UnitsOnOrder > 0, Boolean)))
               Select o

	  ObjectDumper.Write(query)
	End Sub

	<Category("Restriction Operators"), Title("Count - Related Entities Aggregate"), Description("This sample uses COUNT to get Products sold to Customers in the same Country as the Products Suppliers, and where all the Products in the order were from the same Country.")>
	Public Sub LinqToEntities10()
      Dim query = From p In Context.Products
                  Where p.OrderDetails.Where(Function(od) od.Order.Customer.Address.Country = p.Supplier.Address.Country).Count() > 2
                  Select p
      'Dim query = From p In Context.Products
      '                 Where p.OrderDetails.Count(Function(od) od.Order.Customer.Address.Country = p.Supplier.Address.Country).Count() > 2
      '                 Select p

	  ObjectDumper.Write(query)
	End Sub

	#End Region

	#Region "Projection Operators"

	<Category("Projection Operators"), Title("Select - Simple 1"), Description("This samples uses SELECT to get all Customers as Entity Objects.")>
	Public Sub LinqToEntities11()
	  Dim query = From c In Context.Customers
	              Select c

	  ObjectDumper.Write(query)
	End Sub

	<Category("Projection Operators"), Title("Select - Simple 2"), Description("This samples uses SELECT to get all Customer Contact Names as Strings.")>
	Public Sub LinqToEntities12()
	  Dim query = From c In Context.Customers
	              Select c.ContactName

	  ObjectDumper.Write(query)
	End Sub

	<Category("Projection Operators"), Title("Select - Anonymous 1"), Description("This samples uses SELECT to get all Customer Contact Names as an anonoymous type.")>
	Public Sub LinqToEntities13()
	  Dim query = From c In Context.Customers
	              Select New With {Key c.ContactName}

	  ObjectDumper.Write(query)
	End Sub

	<Category("Projection Operators"), Title("Select - Anonymous 2"), Description("This sample uses SELECT to get Orders as anonymous type")>
	Public Sub LinqToEntities14()
	  Dim query = From o In Context.Orders
	              Where o.Customer.Address.City = "London"
	              Select New With {Key o}

	  ObjectDumper.Write(query)
	End Sub

	<Category("Projection Operators"), Title("Select - Anonymous 3"), Description("This sample uses SELECT to get all Orders and associated Customers as anonymous type")>
	Public Sub LinqToEntities15()
	  Dim query = From o In Context.Orders
	              Where o.Customer.Address.City = "London"
	              Select New With {Key o, Key o.Customer}

	  ObjectDumper.Write(query, 1, 3)
	End Sub

	<Category("Projection Operators"), Title("SelectMany - Simple 1"), Description("This sample uses SELECTMANY to get all Orders for a Customer as a flat result")>
	Public Sub LinqToEntities16()
	  Dim query = From c In Context.Customers
	              Where c.CustomerID = "ALFKI"
	              From o In c.Orders
	              Select o

	  ObjectDumper.Write(query)
	End Sub

	<Category("Projection Operators"), Title("SelectMany - Simple 2"), Description("This sample uses SELECTMANY to get all Orders for a Customer as a flat result as a method query")>
	Public Sub LinqToEntities17()
	  Dim query = Context.Customers.Where(Function(cust) cust.CustomerID = "ALFKI").SelectMany(Function(cust) cust.Orders)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Projection Operators"), Title("SelectMany - Simple 3"), Description("This sample uses SELECTMANY to get all Orders for Customers in Denmark as a flat result")>
	Public Sub LinqToEntities18()
	  Dim query = From c In Context.Customers
	              Where c.Address.Country = "Denmark"
	              From o In c.Orders
	              Select o

	  ObjectDumper.Write(query)
	End Sub

	<Category("Projection Operators"), Title("SelectMany - Simple 4"), Description("This sample uses SELECTMANY to get all Orders for Customers in Denmark as a flat result as a method query")>
	Public Sub LinqToEntities19()
	  Dim query = Context.Customers.Where(Function(cust) cust.Address.Country = "Denmark").SelectMany(Function(cust) cust.Orders)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Projection Operators"), Title("SelectMany - Predicate 1"), Description("This sample uses SELECTMANY to get all Orders for Customers in Denmark as a flat result")>
	Public Sub LinqToEntities20()
	  Dim query = From c In Context.Customers
	              Where c.Address.Country = "Denmark"
	              From o In c.Orders
	              Where o.Freight > 5
	              Select o

	  ObjectDumper.Write(query)
	End Sub

	<Category("Projection Operators"), Title("SelectMany - Predicate 2"), Description("This sample uses SELECTMANY to get all Orders for Customers in Denmark as an anonymous type containing the Orders and Customer flat result")>
	Public Sub LinqToEntities21()
	  Dim query = From c In Context.Customers
	              Where c.Address.Country = "Denmark"
	              From o In c.Orders
	              Where o.Freight > 5
	              Select New With {Key c, Key o}

	  ObjectDumper.Write(query, 1)
	End Sub

	<Category("Projection Operators"), Title("SelectMany - Predicate 3"), Description("This sample uses SELECTMANY to get all Orders for Customers in Denmark as a flat result using LINQ opeartors")>
	Public Sub LinqToEntities22()
      Dim query = Context.Customers.Where(Function(cust) cust.Address.Country = "Denmark").SelectMany(Function(cust) cust.Orders.Where(Function(o) CType(o.Freight > 5, Boolean)))

	  ObjectDumper.Write(query)
	End Sub

	#End Region

	#Region "Aggregate Operators"

	<Category("Aggregate Operators"), Title("Count - Simple"), Description("This sample uses COUNT to get the number of Orders.")>
	Public Sub LinqToEntities23()
	  Dim query = Context.Orders.Count()

	  ObjectDumper.Write(query)
	End Sub

	<Category("Aggregate Operators"), Title("Count - Predicate 1"), Description("This sample uses COUNT to get the number of Orders placed by Customers in Mexico.")>
	Public Sub LinqToEntities24()
	  Dim query = Context.Orders.Where(Function(o) o.Customer.Address.Country = "Mexico").Count()

	  ObjectDumper.Write(query)
	End Sub

	<Category("Aggregate Operators"), Title("Count - Predicate 2"), Description("This sample uses COUNT to get the number of Orders shipped to Mexico.")>
	Public Sub LinqToEntities25()
	  Dim query = Context.Orders.Where(Function(o) o.ShipCountry = "Mexico").Count()

	  ObjectDumper.Write(query)
	End Sub

	<Category("Aggregate Operators"), Title("Sum - Simple 1"), Description("This sample uses SUM to find the total freight over all Orders.")>
	Public Sub LinqToEntities26()
	  Dim query = Context.Orders.Select(Function(o) o.Freight).Sum()

	  ObjectDumper.Write(query)
	End Sub

	<Category("Aggregate Operators"), Title("Sum - Simple 2"), Description("This sample uses SUM to find the total number of units on order over all Products.")>
	Public Sub LinqToEntities27()
	  Dim query = Context.Products.Sum(Function(p) p.UnitsOnOrder)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Aggregate Operators"), Title("Sum - Simple 3"), Description("This sample uses SUM to find the total number of units on order over all Products out-of-stock.")>
	Public Sub LinqToEntities28()
      Dim query = Context.Products.Where(Function(p) CType(p.UnitsInStock = 0, Boolean)).Sum(Function(p) p.UnitsOnOrder)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Aggregate Operators"), Title("Min - Simple 1"), Description("This sample uses MIN to find the lowest unit price of any Product.")>
	Public Sub LinqToEntities29()
	  Dim query = Context.Products.Select(Function(p) p.UnitPrice).Min()

	  ObjectDumper.Write(query)
	End Sub

	<Category("Aggregate Operators"), Title("Min - Simple 2"), Description("This sample uses MIN to find the lowest freight of any Order.")>
	Public Sub LinqToEntities30()
	  Dim query = Context.Orders.Min(Function(o) o.Freight)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Aggregate Operators"), Title("Min - Predicate"), Description("This sample uses MIN to find the lowest freight of any Order shipped to Mexico.")>
	Public Sub LinqToEntities31()
	  Dim query = Context.Orders.Where(Function(o) o.ShipCountry = "Mexico").Min(Function(o) o.Freight)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Aggregate Operators"), Title("Min - Grouping"), Description("This sample uses Min to find the Products that have the lowest unit price in each category, and returns the result as an anonoymous type.")>
	Public Sub LinqToEntities32()
	  Dim query = From p In Context.Products
	              Group p By p.Category.CategoryID Into g = Group
	              Select New With {Key .CategoryID = CategoryID, Key .CheapestProducts =
	                  From p2 In g
	                  Where p2.UnitPrice = g.Min(Function(p3) p3.UnitPrice)
	                  Select p2}

	  ObjectDumper.Write(query, 1)
	End Sub

	<Category("Aggregate Operators"), Title("Max - Simple 1"), Description("This sample uses MAX to find the latest hire date of any Employee.")>
	Public Sub LinqToEntities33()
	  Dim query = Context.Employees.Select(Function(e) e.HireDate).Max()

	  ObjectDumper.Write(query)
	End Sub

	<Category("Aggregate Operators"), Title("Max - Simple 2"), Description("This sample uses MAX to find the most units in stock of any Product.")>
	Public Sub LinqToEntities34()
	  Dim query = Context.Products.Max(Function(p) p.UnitsInStock)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Aggregate Operators"), Title("Max - Predicate"), Description("This sample uses MAX to find the most units in stock of any Product with CategoryID = 1.")>
	Public Sub LinqToEntities35()
	  Dim query = Context.Products.Where(Function(p) p.Category.CategoryID = 1).Max(Function(p) p.UnitsInStock)
	  ObjectDumper.Write(query)
	End Sub

	<Category("Aggregate Operators"), Title("Max - Grouping"), Description("This sample uses MAX to find the Products that have the highest unit price in each category, and returns the result as an anonoymous type.")>
	Public Sub LinqToEntities36()
	  Dim query = From p In Context.Products
	              Group p By p.Category.CategoryID Into g = Group
	              Select New With {Key CategoryID, Key .MostExpensiveProducts =
	                  From p2 In g
	                  Where p2.UnitPrice = g.Max(Function(p3) p3.UnitPrice)
	                  Select p2}

	  ObjectDumper.Write(query, 1)
	End Sub

	<Category("Aggregate Operators"), Title("Average - Simple 1"), Description("This sample uses AVERAGE to find the average freight of all Orders.")>
	Public Sub LinqToEntities37()
	  Dim query = Context.Orders.Select(Function(o) o.Freight).Average()

	  ObjectDumper.Write(query)
	End Sub

	<Category("Aggregate Operators"), Title("Average - Simple 2"), Description("This sample uses AVERAGE to find the average unit price of all Products.")>
	Public Sub LinqToEntities38()
	  Dim query = Context.Products.Average(Function(p) p.UnitPrice)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Aggregate Operators"), Title("Average - Predicate"), Description("This sample uses AVERAGE to find the average unit price of all Products with CategoryID = 1.")>
	Public Sub LinqToEntities39()
	  Dim query = Context.Products.Where(Function(p) p.Category.CategoryID = 1).Average(Function(p) p.UnitPrice)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Aggregate Operators"), Title("Average - Grouping 1"), Description("This sample uses AVERAGE to find the Products that have unit price higher than the average unit price of the category for each category.")>
	Public Sub LinqToEntities40()
	  Dim query = From p In Context.Products
	              Group p By p.Category.CategoryID Into g = Group
	              Select New With {Key CategoryID, Key .ExpensiveProducts =
	                  From p2 In g
	                  Where p2.UnitPrice > g.Average(Function(p3) p3.UnitPrice)
	                  Select p2}

	  ObjectDumper.Write(query, 2)
	End Sub

	<Category("Aggregate Operators"), Title("Average - Grouping 2"), Description("This sample uses AVERAGE to find the average unit price of each category.")>
	Public Sub LinqToEntities41()
	  Dim query = From p In Context.Products
	              Group p By p.Category.CategoryID Into g = Group
	              Select New With {Key CategoryID, Key .Average = g.Average(Function(p) p.UnitPrice)}

	  ObjectDumper.Write(query)
	End Sub

	#End Region

	#Region "Set Operators"

	<Category("Set Operators"), Title("First - Simple"), Description("This sample uses FIRST and WHERE to get the first (database order) order that is shipped to Seattle. The WHERE predicate is evaluated on the server.")>
	Public Sub LinqToEntities42()
	  Dim query = From o In Context.Orders
	              Where o.ShipCity = "Seattle"
	              Select o

	  Dim result = query.First()

	  ObjectDumper.Write(result)
	End Sub

	<Category("Set Operators"), Title("First - Predicate"), Description("This sample uses FIRST to get the first (database order) order that is shipped to Seattle. The predicate is evaluated on the client.")>
	Public Sub LinqToEntities43()
	  Dim query = From o In Context.Orders
	              Select o

	  Dim result = query.First(Function(x) x.ShipCity = "Seattle")

	  ObjectDumper.Write(result)
	End Sub

	<Category("Set Operators"), Title("First - Ordered"), Description("This sample uses FIRST, WHERE and ORDER BY to get the first order that is shipped to Seattle, ordered by date. The predicate is evaluated on the server.")>
	Public Sub LinqToEntities44()
	  Dim query = From o In Context.Orders
	              Where o.ShipCity = "Seattle"
	              Order By o.OrderDate
	              Select o

	  Dim result = query.First()

	  ObjectDumper.Write(result)
	End Sub

	<Category("Set Operators"), Title("Distinct - Simple"), Description("This sample uses DISTINCT to get all the categories of products.")>
	Public Sub LinqToEntities45()
	  Dim query = Context.Products.Select(Function(o) o.Category).Distinct()

	  ObjectDumper.Write(query)
	End Sub

	<Category("Set Operators"), Title("Union - Simple"), Description("This sample uses UNION to get all the orders where the shipping country was Mexico or Canada.")>
	Public Sub LinqToEntities46()
	  Dim mexico = Context.Orders.Where(Function(o) o.ShipCountry = "Mexico").Select(Function(o) o)
	  Dim canada = Context.Orders.Where(Function(o) o.ShipCountry = "Canada").Select(Function(o) o)
	  Dim query = mexico.Union(canada)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Set Operators"), Title("Union - With Distinct"), Description("This sample uses UNION and DISTINCT to get all the Customers from orders where the shipping country was Mexico or Canada.")>
	Public Sub LinqToEntities47()
	  Dim mexico = Context.Orders.Where(Function(o) o.ShipCountry = "Mexico").Select(Function(o) o)
	  Dim canada = Context.Orders.Where(Function(o) o.ShipCountry = "Canada").Select(Function(o) o)
	  Dim union = mexico.Union(canada).Select(Function(o) o.Customer)

	  Dim query = union.Distinct()

	  ObjectDumper.Write(query)
	End Sub

	<Category("Set Operators"), Title("Concat - Simple"), Description("This sample uses CONCAT to get all orders where the shipping country was Mexico or Canada.")>
	Public Sub LinqToEntities48()
	  Dim mexico = Context.Orders.Where(Function(o) o.ShipCountry = "Mexico").Select(Function(o) o)
	  Dim canada = Context.Orders.Where(Function(o) o.ShipCountry = "Canada").Select(Function(o) o)

	  Dim query = mexico.Concat(canada)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Set Operators"), Title("Intersect - Simple 1"), Description("This sample uses INTERSECT to get common products where an order was shipped to Mexico or Canada.")>
	Public Sub LinqToEntities49()
	  Dim mexico = Context.OrderDetails.Where(Function(od) od.Order.ShipCountry = "Mexico").Select(Function(od) od.Product)
	  Dim canada = Context.OrderDetails.Where(Function(od) od.Order.ShipCountry = "Canada").Select(Function(od) od.Product)

	  Dim query = mexico.Intersect(canada)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Set Operators"), Title("Intersect - Simple 2"), Description("This sample uses INTERSECT to get common products where an order was shipped to Mexico or USA in one consolidated query.")>
	Public Sub LinqToEntities50()
	  Dim query = Context.OrderDetails.Where(Function(od) od.Order.ShipCountry = "Mexico").Select(Function(od) od.Product).Intersect(Context.OrderDetails.Where(Function(od) od.Order.ShipCountry = "USA").Select(Function(o) o.Product))

	  ObjectDumper.Write(query)
	End Sub

	<Category("Set Operators"), Title("Except - Simple 1"), Description("This sample uses EXCEPT to get customers who shipped orders to Mexico but not Canada.")>
	Public Sub LinqToEntities51()
	  Dim query = Context.Orders.Where(Function(o) o.ShipCountry = "Mexico").Select(Function(o) o.Customer).Except(Context.Orders.Where(Function(o) o.ShipCountry = "Canada").Select(Function(o) o.Customer))

	  ObjectDumper.Write(query)
	End Sub

	<Category("Set Operators"), Title("Except - Simple 2"), Description("This sample uses EXCEPT to get customers with no orders sent to Mexico.")>
	Public Sub LinqToEntities52()
	  Dim query = Context.Customers.Select(Function(e) e).Except(Context.Orders.Where(Function(o) o.ShipCountry = "Mexico").Select(Function(o) o.Customer))

	  ObjectDumper.Write(query)
	End Sub

	#End Region

	#Region "Ordering and Grouping"

	<Category("Ordering and Grouping"), Title("OrderBy - Simple 1"), Description("Select all customers ordered by ContactName.")>
	Public Sub LinqToEntities53()
	  Dim query = From c In Context.Customers
	              Order By c.ContactName
	              Select c

	  ObjectDumper.Write(query)
	End Sub

	<Category("Ordering and Grouping"), Title("OrderBy - Simple 2"), Description("Select all customers ordered by ContactName descending.")>
	Public Sub LinqToEntities54()
	  Dim query = From c In Context.Customers
	              Order By c.CompanyName Descending
	              Select c

	  ObjectDumper.Write(query)
	End Sub

	<Category("Ordering and Grouping"), Title("OrderBy - Simple 3"), Description("Select an anonoymous type with all product IDs ordered by UnitsInStock.")>
	Public Sub LinqToEntities55()
	  Dim query = From p In Context.Products
	              Order By p.UnitsInStock
	              Select New With {Key p.ProductID, Key p.UnitsInStock}

	  ObjectDumper.Write(query)
	End Sub

	<Category("Ordering and Grouping"), Title("OrderBy - Simple 4"), Description("Select an anonoymous type with all product IDs ordered by UnitsInStock as a method query.")>
	Public Sub LinqToEntities56()
	  Dim query = Context.Products.OrderBy(Function(p) p.UnitsInStock).Select(Function(p2) New With {Key p2.ProductID, Key p2.UnitsInStock})

	  ObjectDumper.Write(query)
	End Sub

	<Category("Ordering and Grouping"), Title("OrderByDescending - Simple 1"), Description("Select all customers ordered by the descending region.")>
	Public Sub LinqToEntities57()
	  Dim query = From c In Context.Customers
	              Order By c.Address.Region Descending
	              Select c

	  ObjectDumper.Write(query)
	End Sub

	<Category("Ordering and Grouping"), Title("OrderByDescending - Simple 2"), Description("Select all customers ordered by the descending region as a method query.")>
	Public Sub LinqToEntities58()
	  Dim query = Context.Customers.Select(Function(c) c).OrderByDescending(Function(c2) c2.Address.Region)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Ordering and Grouping"), Title("OrderBy with ThenBy"), Description("Select all customers ordered by the region, then the contact name.")>
	Public Sub LinqToEntities59()
	  Dim query = Context.Customers.Select(Function(c) c).OrderBy(Function(c) c.Address.Region).ThenBy(Function(c) c.ContactName)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Ordering and Grouping"), Title("OrderByDescending with ThenBy"), Description("Select all customers ordered by the region in descending order, then the contact name.")>
	Public Sub LinqToEntities60()
	  Dim query = Context.Customers.Select(Function(c) c).OrderByDescending(Function(c) c.Address.Region).ThenBy(Function(c) c.ContactName)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Ordering and Grouping"), Title("OrderBy with ThenByDescending"), Description("Select all customers ordered by the region then the contact name in descending order.")>
	Public Sub LinqToEntities61()
	  Dim query = Context.Customers.Select(Function(c) c).OrderBy(Function(c) c.Address.Region).ThenByDescending(Function(c) c.ContactName)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Ordering and Grouping"), Title("OrderByDescending - Simple 3"), Description("Select all products ordered by the descending unit price.")>
	Public Sub LinqToEntities62()
	  Dim query = From p In Context.Products
	              Order By p.UnitPrice Descending
	              Select p

	  ObjectDumper.Write(query)
	End Sub

	<Category("Ordering and Grouping"), Title("OrderBy - FK Collection"), Description("Select all orders for a customer ordered by date that the order was placed.")>
	Public Sub LinqToEntities63()
	  Dim query = Context.Customers.Where(Function(cust) cust.CustomerID = "ALFKI").SelectMany(Function(c) c.Orders.Select(Function(o) o)).OrderBy(Function(o2) o2.OrderDate)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Ordering and Grouping"), Title("Grouping - Simple 1"), Description("Select all Regions with a customer.")>
	Public Sub LinqToEntities64()
	  Dim query = From c In Context.Customers
	              Group c By c.Address.Region Into regions = Group
	              Select New With {Key Region}

	  ObjectDumper.Write(query)
	End Sub

	<Category("Ordering and Grouping"), Title("Grouping - Simple 2"), Description("Select all dates with orders placed.")>
	Public Sub LinqToEntities65()
	  Dim query = From o In Context.Orders
	              Group o By o.OrderDate Into dates = Group
	              Select New With {Key OrderDate}

	  ObjectDumper.Write(query)
	End Sub

	<Category("Ordering and Grouping"), Title("Grouping - Join 1"), Description("Select all Regions and customer count for each region.")>
	Public Sub LinqToEntities66()
	  Dim query = From c In Context.Customers
	              Group c By c.Address.Region Into regions = Group
	              Select New With {Key .region = Region, Key .count = regions.Count()}

	  ObjectDumper.Write(query)
	End Sub

	<Category("Ordering and Grouping"), Title("Grouping - Join 2"), Description("Select all Regions and customer count for each region as a method query.")>
	Public Sub LinqToEntities67()
	  Dim query = Context.Customers.GroupBy(Function(c) c.Address.Region).Select(Function(r) New With {Key .region = r.Key, Key .count = r.Count()})

	  ObjectDumper.Write(query)
	End Sub

	<Category("Ordering and Grouping"), Title("Grouping with a join on Key 1"), Description("Select all Customer Regions with the total Freight on all orders for Customers in that Region.")>
	Public Sub LinqToEntities68()
	  Dim query = From c In Context.Customers
	              Group c By c.Address.Region Into regions = Group Join c2 In Context.Customers On Region Equals c2.Address.Region
	              Select New With {Key .region = Region, Key .total = c2.Orders.Sum(Function(o) o.Freight)}

	  ObjectDumper.Write(query)
	End Sub

	<Category("Ordering and Grouping"), Title("Grouping with a join on Key 2"), Description("Select all Customer Regions with the total Freight on all orders for Customers in that Region as a method query.")>
	Public Sub LinqToEntities69()
	  Dim query = Context.Customers.GroupBy(Function(c) c.Address.Region).Select(Function(g) New With {Key .Region = g.Key, Key .FreightTotal = g.SelectMany(Function(c2) c2.Orders).Sum(Function(o) o.Freight)})

	  ObjectDumper.Write(query)
	End Sub

	#End Region

	#Region "Relationship Navigation"

	<Category("Relationship Navigation"), Title("Relationship Collection 1"), Description("Select a sequence of all the orders for a customer using Select.")>
	Public Sub LinqToEntities70()
	  Dim query = Context.Customers.Where(Function(cust) cust.CustomerID = "ALFKI").Select(Function(c) c.Orders)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Relationship Navigation"), Title("Relationship Collection 2"), Description("Select all the orders for a customer using SelectMany.")>
	Public Sub LinqToEntities71()
	  Dim query = Context.Customers.Where(Function(cust) cust.CustomerID = "ALFKI").SelectMany(Function(c) c.Orders)

	  ObjectDumper.Write(query)
	End Sub


	<Category("Relationship Navigation"), Title("Relationship Collection 3"), Description("Select number of orders placed in 1998 for a customer.")>
	Public Sub LinqToEntities74()
	  Dim query = Context.Customers.Where(Function(cust) cust.CustomerID = "ALFKI").SelectMany(Function(c) c.Orders).Where(Function(o) o.OrderDate.HasValue = True AndAlso o.OrderDate.Value.Year = 1998)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Relationship Navigation"), Title("Relationship Collection Aggregate"), Description("Select a customer and the sum of the freight of thier orders.")>
	Public Sub LinqToEntities73()
	  Dim query = Context.Customers.Where(Function(cust) cust.CustomerID = "ALFKI").Select(Function(c) c.Orders.Sum(Function(o) o.Freight))

	  ObjectDumper.Write(query)
	End Sub

	<Category("Relationship Navigation"), Title("Relationship Collection with Predicate"), Description("Select customers with an order where the shipping address is the same as the customers.")>
	Public Sub LinqToEntities75()
	  Dim query = Context.Customers.Where(Function(cust) cust.Orders.Any(Function(o) o.ShipAddress = cust.Address.Address)).Select(Function(c2) c2)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Relationship Navigation"), Title("Relationship Collection with Grouping and Aggregate"), Description("Selects all regions with a customer, and shows the sum of orders for customers for each region.")>
	Public Sub LinqToEntities76()
	  Dim query = From c In Context.Customers
	              Group c By c.Address.Region Into regions = Group Join c2 In Context.Customers On Region Equals c2.Address.Region
	              Select New With {Key .region = Region, Key .total = c2.Orders.Sum(Function(o) o.Freight)}

	  ObjectDumper.Write(query)
	End Sub

	#End Region

	#Region "Inheritance"

	<Category("Inheritance"), Title("Show Type in Hierarchy"), Description("Select all products, both active and discontinued products, and shows the type.")>
	Public Sub LinqToEntities77()
	  Dim query = Context.Products.Select(Function(p) p)

		' force local execution to show local type
	  Dim query2 = query.AsEnumerable().Select(Function(p) New With {Key .type = p.GetType().ToString(), Key .prod = p})

	  ObjectDumper.Write(query2, 2, 3)
	End Sub

	<Category("Inheritance"), Title("Select by Type in Hierarchy - OfType 1"), Description("Select only discontinued products.")>
	Public Sub LinqToEntities78()
	  Dim query = Context.Products.OfType(Of DiscontinuedProduct)().Select(Function(p) p)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Inheritance"), Title("Select by Type in Hierarchy - OfType 2"), Description("Select only products, which will reutrn all Products and subtypes of Products (DiscontinuedProducts and ActiveProducts).")>
	Public Sub LinqToEntities79()
	  Dim query = Context.Products.OfType(Of Product)().Select(Function(p) p)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Inheritance"), Title("Select by Type in Hierarchy - OfType 3"), Description("Select only discontinued products.")>
	Public Sub LinqToEntities80()
	  Dim query = Context.Products.OfType(Of DiscontinuedProduct)()

	  ObjectDumper.Write(query)
	End Sub

	<Category("Inheritance"), Title("Select by Type in Hierarchy - is"), Description("Select only discontinued products.")>
	Public Sub LinqToEntities81()
	  Dim query = Context.Products.Where(Function(p) TypeOf p Is DiscontinuedProduct)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Inheritance"), Title("Select by Subtype in Hierarchy - OfType"), Description("Select all current employees.")>
	Public Sub LinqToEntities87()
	  Dim query = Context.Employees.OfType(Of CurrentEmployee)().ToList().Select(Function(p) New With {Key .type = p.GetType().ToString(), Key p})

	  ObjectDumper.Write(query, 2, 3)
	End Sub

	#End Region

	#Region "Runtime behavior closure"

	Private Class [MyClass]
	  Public Shared Val As Decimal = 50

	  Public Function GetVal() As Decimal
        Return 50
	  End Function
	End Class

	<Category("Runtime behavior example"), Title("Static variable reference"), Description("Uses a local variable as a query parameter.")>
	Public Sub LinqToEntities91()
      Dim c As New [MyClass]()
      'Dim query = Context.Orders.Where(Function(o) o.Freight > MyClass.Val).Select(Function(o) o)
      Dim query = Context.Orders.Where(Function(o) CType(o.Freight > 50, Boolean)).Select(Function(o) o)

	  ObjectDumper.Write(query, 1)
	End Sub

	<Category("Runtime behavior example"), Title("Deferred execution"), Description("Uses a the value of the local variable at query execution time.")>
	Public Sub LinqToEntities92()
	  Dim x As Decimal = 50
      Dim query = Context.Orders.Where(Function(o) CType(o.Freight > x, Boolean)).Select(Function(o) New With {Key o.Freight, Key o})
	  x = 100
	  ObjectDumper.Write(query)
	End Sub

	<Category("Runtime behavior example"), Title("Closure"), Description("Each execution uses the current value of the local variable.")>
	Public Sub LinqToEntities93()
	  Dim x As Decimal = 100

      Dim query = Context.Orders.Where(Function(o) CType(o.Freight > x, Boolean)).Select(Function(o) New With {Key o.Freight, Key o})

	  ObjectDumper.Write(x)
	  ObjectDumper.Write(query)

	  x = 200
	  ObjectDumper.Write(x)
	  ObjectDumper.Write(query)
	End Sub

	#End Region

	#Region "Span"

	<Category("Span"), Title("Load Related Entities - One Level"), Description("Load OrderDetails with Orders ")>
	Public Sub LinqToEntities94()
	  ' var query = Context.Orders.Include("OrderDetails").Where(c => c.Customer.Address.City == "London").Select(o => o);
	  Dim query = Context.Orders.Where(Function(c) c.Customer.Address.City = "London").Select(Function(o) o).Include("OrderDetails")

	  ObjectDumper.Write(query, 1)
	End Sub

	<Category("Span"), Title("Load Related Entities - Two Levels"), Description("Load OrderDetails and Products with Orders ")>
	Public Sub LinqToEntities95()
	  Dim query = Context.Orders.Take(3).Select(Function(o) o).Include("OrderDetails").Include("OrderDetails.Product")

	  ObjectDumper.Write(query, 2)
	End Sub

	#End Region

	#Region "Paging"

	<Category("Paging"), Title("Skip 1"), Description("Skip the most recent 2 orders from customers in London")>
	Public Sub LinqToEntities96()
	  Dim query = Context.Orders.Where(Function(o) o.Customer.Address.City = "London").OrderBy(Function(o) o.OrderDate).Skip(2).Select(Function(o) o)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Paging"), Title("Take 1"), Description("Take the 2 most recent Orders ")>
	Public Sub LinqToEntities97()
	  Dim query = Context.Orders.OrderBy(Function(o) o.OrderDate).Take(2).Select(Function(o) o)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Paging"), Title("Take and Skip - 1"), Description("Take the 10th to the 20th Orders, ordered by date ")>
	Public Sub LinqToEntities98()
	  Dim query = Context.Orders.OrderBy(Function(o) o.OrderDate).Skip(10).Take(10).Select(Function(o) o)

	  ObjectDumper.Write(query)
	End Sub

	<Category("Paging"), Title("Take and Skip - 2"), Description("Use a page number variable to get the xth page")>
	Public Sub LinqToEntities99()
	  Dim pageSize As Integer = 10
	  Dim pageNumber As Integer = 4

	  Dim query = Context.Orders.OrderBy(Function(o) o.OrderDate).Skip(pageSize * pageNumber).Take(pageSize).Select(Function(o) o)
	  ObjectDumper.Write(query)
	End Sub

	#End Region

	#Region "Compiled Queries"

	'[Category("Compiled Queries")]
	'[Title("Simple")]
	'[Description("This sample uses WHERE in a compiled query to find all customers whose contact title is Sales Representative.")]
	'public void LinqToEntities100()
	'{

	'    var cq = CompiledQuery.Compile<NorthwindContext, IQueryable<Customer>>(
	'        ctx =>
	'        from cust in ctx.Customers
	'        where cust.ContactTitle == "Sales Representative"
	'        select cust);

	'    ObjectDumper.Write(cq(context));
	'}

	'[Category("Compiled Queries")]
	'[Title("Anonymous Type Result")]
	'[Description("This sample uses WHERE in a compiled query to find all customers whose contact title is Sales Representative. The result type is an anonymous type, so the non-generic form of CompiledQuery.Compile() is used.")]
	'public void LinqToEntities101()
	'{

	'    var cq = CompiledQuery.Compile(
	'        (NorthwindContext ctx) =>
	'        from cust in ctx.Customers
	'        where cust.ContactTitle == "Sales Representative"
	'        select new { cust.CustomerID, cust.ContactName });

	'    ObjectDumper.Write(cq(context));
	'}

	'[Category("Compiled Queries")]
	'[Title("Parametrized")]
	'[Description("This sample uses WHERE in a parametrized compiled query to find all orders placed before 1997.")]
	'public void LinqToEntities102()
	'{
	'    var cq = CompiledQuery.Compile(
	'        (DomainModel.NorthwindContext ctx, DateTime minOrderDate) =>
	'        from order in ctx.Orders
	'        where order.OrderDate < minOrderDate
	'        select order);

	'    ObjectDumper.Write(cq(context, new DateTime(1997, 1, 1)));
	'}

	#End Region

	#Region "Public Methods"
	Public Overrides Sub LoadCache()

	  'Load customers and related objects into cache
	  Context.Connect()
	  Dim query = Context.Customers.Include("Orders").Include("Orders.OrderDetails").Include("Orders.OrderDetails.Product").Include("Orders.OrderDetails.Product.Supplier")
	  query.QueryStrategy = QueryStrategy.DataSourceOnly
	  query.ToList()
	End Sub
	#End Region ' Public Methods

  End Class
End Namespace

